// Generated by CoffeeScript 2.4.1
// ## A Brief Tour of dyna3

// When you load dyna3, you should initially see a three-dimensional coordinate system with labeled axes.
// ```javascript
var LTx, axes, buildAxis, main;

import {
  threeLoaded,
  three as J3,
  TrackballControls as J3x
} from './externals.js';

LTx = {
  dashed: true,
  plain: false
};

buildAxis = function(to, color, tx) {
  var axis, geom, mat;
  geom = new J3.Geometry();
  if (tx) {
    mat = new J3.LineDashedMaterial({
      linewidth: 3,
      color,
      dashSize: 0.5,
      gapSize: 0.5
    });
  } else {
    mat = new J3.LineBasicMaterial({
      linewidth: 3,
      color
    });
  }
  geom.vertices.push(new J3.Vector3(0, 0, 0));
  geom.vertices.push(to.clone());
  axis = new J3.Line(geom, mat, J3.LineSegments);
  axis.computeLineDistances();
  return axis;
};

axes = function(length) {
  var ax, cols, dirs, i, j, len, ref;
  ax = new J3.Object3D();
  dirs = [new J3.Vector3(length, 0, 0), new J3.Vector3(0, length, 0), new J3.Vector3(0, 0, length)];
  cols = [0xFF0000, 0x00FF00, 0x0000FF];
  ref = [0, 1, 2];
  for (j = 0, len = ref.length; j < len; j++) {
    i = ref[j];
    ax.add(buildAxis(dirs[i], cols[i], LTx.plain));
    dirs[i].multiplyScalar(-1);
    ax.add(buildAxis(dirs[i], cols[i], LTx.dashed));
  }
  return ax;
};

main = function() {
  var ambient, ball, camera, controls, geometry, material, renderer, rht, rwd, scene;
  renderer = new J3.WebGLRenderer();
  rwd = window.innerWidth;
  rht = window.innerHeight;
  renderer.setSize(rwd, rht);
  renderer.setClearColor(0xdddddd, 1.0);
  document.body.appendChild(renderer.domElement);
  scene = new J3.Scene();
  geometry = new J3.IcosahedronBufferGeometry(1);
  material = new J3.MeshPhongMaterial({
    color: 0xff00ff
  });
  ball = new J3.Mesh(geometry, material);
  scene.add(ball);
  scene.add(axes(10));
  ambient = new J3.HemisphereLight(0xffffff, 0xaaaaaa, 1);
  ambient.position.set(0.1, 0.1, 1);
  scene.add(ambient);
  camera = new J3.PerspectiveCamera(75, rwd / rht, 0.1, 1000);
  camera.up.set(0, 0, 1);
  camera.position.set(4, -1, 3);
  controls = new J3x.TrackballControls(camera, renderer.domElement);
  return renderer.setAnimationLoop(function() {
    controls.update();
    return renderer.render(scene, camera);
  });
};

threeLoaded.then(main);

// ```

//# sourceMappingURL=dyna3.js.map
